library S485IIC;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  SysUtils,
  Classes,
  windows,
  OoMisc,
  AdPort,
  extctrls,
  Messages,
  Dialogs;

//----------Windows Messages String------------------------
Const MSGReadI2CTOut   = $01;
      MSGWriteI2CTOut  = $02;
      MSGReadResponse  = $03;
      MSGWriteResponse = $04;
      MSGIO1           = $05;
      MSGIO2           = $06;
      MSGSCLFreq       = $07;
      MSGSCLChange     = $08;
      MSGComOpen       = $09;
      MSGComClose      = $0A;
      MSGDVer          = $0B;
      MSGCSlave        = $0C;
      MSGCSErr         = $0D;
      MSGIO            = $0E;
      MSGUnknownCMD    = $FF;


      I2Cw          = '77'; //w
      I2Cr          = '72'; //r
      I2Cd          = 'd';
      I2CCheck      = '63'; //c
      I2CFreqChange = '65'; //e
      I2CFreqReq    = '69'; //i
      I2CIO1        = '6D'; //m
      I2CIO2        = '6E'; //n
      I2CIO         = '6F'; //o
      I2CVerNr      = '76'; //v
      I2CCSFehler   = 's';

      szPrivateMsgString : PChar = 'SerIIC_DLL32_MSG_String';

Type
     TI2CData = Record
       iDIP    : Byte;
       iSlvAdr : Byte;
       iData   : Array[0..127] of Byte;
       iLen    : Byte;
     end;

     TApdComPort1 = Class(TApdComPort)
     Private
     Public
       Procedure ApdComPortTriggerAvail(CP: TObject; Count: Word);
       procedure ApdComPortPortClose(Sender: TObject);
       procedure ApdComPortPortOpen(Sender: TObject);
     end;

     TTimer1   = Class(TTimer)
     private
     Public
       Procedure Timer(Sender : TObject);
     end;

     TTimer2   = Class(TTimer)
     private
     Public
       Procedure Timer(Sender : TObject);
     end;

Var wPrivateMsg  : Cardinal;
    ApdComPort   : TApdComPort1;
    ReadI2CTimer : TTimer1;
    WriteI2CTimer : TTimer2;
    CR_Bool   : Boolean;
    Counter   : Integer;
    ucChr     : Array[0..1024] of Char;
    CMD       : Integer;
    I2CReadB   : Boolean;
    I2CReadStr : String;
    I2CBusy    : Boolean;
    I2CData    : TI2CData;
    I2SCLL     : Byte;
    I2SCLH     : Byte;
    MPCLK      : Integer;
    SCLFreq    : Word;
    SlvAdr     : Byte;
    DevVer     : PChar;
    DIPSwitch  : Byte;


function BroadcastSystemMessage(Flags: DWORD; Recipients: PDWORD;
                                uiMessage: UINT; wParam: WPARAM;
                                lParam: LPARAM): Longint; stdcall;
         external user32 name 'BroadcastSystemMessage';

procedure SendMSG(wMsgValue, lMsgValue : LongInt);
var dwRecipient : DWord;
begin
  // Botschaft abschicken
  dwRecipient := BSM_APPLICATIONS;
  //BSF_FLUSHDISK
  //BSF_FORCEIFHUNG
  //BSF_NOHANG
  //BSF_NOTIMEOUTIFNOTHUNG
  //BSF_POSTMESSAGE
  BroadcastSystemMessage(BSF_POSTMESSAGE OR BSF_FLUSHDISK, @dwRecipient,
                         wPrivateMsg, wMsgValue, lMsgValue);
end;

function SendStr(Str: String): Boolean;
var i      : Integer;
    ChkSum : Byte;
begin
  ChkSum :=0;
  for i := 1 to Length(Str) do
    ChkSum := ChkSum + Ord(Str[i]);
  ChkSum := $100 - (ChkSum Mod 256);
  Str := Str + IntToHex(ChkSum,2);
  Str := Str + #13;

  if ApdComPort.Open then
  begin
    ApdComport.PutString(Str);
    Result := true;
  end else Result := false;
end;

function GetChkSum(Stri : String): Boolean;
var i      : Integer;
    ChkSum, vChkSum : Byte;
begin
  vChkSum :=0;
  for i := 1 to Length(Stri) - 2 do
    vChkSum := vChkSum + Ord(Stri[i]);
  vChkSum := $100 - (vChkSum Mod 256);

  ChkSum := StrToInt('$'+Copy(Stri,Length(Stri)-1,2));
  Result := (vChkSum = ChkSum);
end;

function SerI2CCreate : Cardinal; stdcall;
begin
  I2SCLH     := 7;
  I2SCLL     := 8;
  MPCLK      := 12000000;
  ApdComPort := TApdComPort1.Create(nil);
  with ApdComPort do
  begin
    AutoOpen      := true;
    Baud          := 19200;
    BufferFull    := 0;
    BufferResume  := 0;
    ApdComPort.CommNotificationLevel := 10;
    ComNumber     := 1;
    DataBits      := 8;
    DeviceLayer   := dlWin32;
    DTR           := true;
    InSize        := 4096;
    LogAllHex     := false;
    Logging       := tlOff;
    LogName       := '';
    LogSize       := 10000;
    OutSize       := 4096;
    Parity        := pNone;
    PromptForPort := true;
    RS485Mode     := false;
    RTS           := true;
    StopBits      := 1;
    SWFlowOptions := swfNone;
    TapiMode      := tmAuto;
    TraceAllHex   := false;
    TraceHex      := false;
    TraceName     := '';
    TraceSize     := 10000;
    Tracing       := tlOff;
    UseEventWord  := true;
    UseMSRShadow  := true;
    XOffChar      := #19;
    XOnChar       := #17;
    OnPortOpen    := ApdComPortPortOpen;
    OnPortClose   := ApdComPortPortClose;
    OnTriggerAvail := ApdComPortTriggerAvail;
  end;

  ReadI2CTimer := TTimer1.Create(nil);
  with ReadI2CTimer do
  begin
    Enabled  := false;
    Interval := 1500;
    OnTimer  := Timer;
  end;

  WriteI2CTimer := TTimer2.Create(nil);
  with WriteI2CTimer do
  begin
    Enabled  := false;
    Interval := 1500;
    OnTimer  := Timer;
  end;
  wPrivateMsg := 0;
  wPrivateMsg := RegisterWindowMessage(szPrivateMsgString);
  Result := wPrivateMsg;
end;

Function SerI2CDestroy : Byte; stdcall;
begin
  try
    ApdComPort.Open := false;
    ApdComPort.Free;
    ReadI2CTimer.Free;
    WriteI2CTimer.Free;
    Result := 1;
  except
    Result := 0;
  end;
end;

Procedure TTimer1.Timer(Sender : TObject);
var lPrm : LongInt;
begin
  ReadI2CTimer.Enabled := false;
  I2CBusy := false;
  lPrm := SlvAdr;
  lPrm := lPrm Shl 16;
  SendMSG(MSGReadI2CTOut,lPrm);
end;

function SetReadTOut(TiOut : Word): Word; stdcall;
begin
  if TiOut > 0 then
  begin
    ReadI2CTimer.Interval := TiOut;
    Result := ReadI2CTimer.Interval;
  end else Result := 0;
end;

function GetReadTOut : Word; stdcall;
begin
  Result := ReadI2CTimer.Interval;
end;

Procedure TTimer2.Timer(Sender : TObject);
var lPrm : LongInt;
begin
  WriteI2CTimer.Enabled := false;
  I2CBusy := false;
  lPrm := SlvAdr;
  lPrm := lPrm Shl 16;
  SendMSG(MSGWriteI2CTOut,lPrm);
end;

function GetWriteTOut : Word; stdcall;
begin
  Result := WriteI2CTimer.Interval;
end;

function SetWriteTOut(TiOut : Word): Word; stdcall;
begin
  if TiOut > 0 then
  begin
    WriteI2CTimer.Interval := TiOut;
    Result := WriteI2CTimer.Interval;
  end else Result := 0;
end;

Function ParseI2CData(I2CStr : String) : Byte;
var i,k : Integer;
    len : Integer;
begin
  Try
    len := StrToInt('$'+copy(I2CStr,5,2));
    SlvAdr := StrToInt('$'+copy(I2CStr,3,2));
    DIPSwitch := StrToInt('$'+copy(I2CStr,1,2));

    k := 7;
    for i := 0 to Len -1 do
    begin
      I2CData.iData[i] := StrToInt('$'+copy(I2CStr,k,2));
      k := k + 2;
    end;

    I2CData.iLen    := len;
    I2CData.iSlvAdr := SlvAdr;
    I2CData.iDIP    := DIPSwitch;
    Result := 1;
  except
    I2CData.iLen    := 0;
    I2CData.iSlvAdr := 0;
    I2CData.iDIP    := 0;
    Result := 0;
  end;
end;

procedure TApdComPort1.ApdComPortPortClose(Sender: TObject);
begin
  SendMSG(MSGComClose,ApdComPort.ComNumber);
end;

procedure TApdComPort1.ApdComPortPortOpen(Sender: TObject);
begin
  SendMSG(MSGComOpen,ApdComPort.ComNumber);
end;

Procedure TApdComPort1.ApdComPortTriggerAvail(CP: TObject; Count: Word);
var I       : Word;
    DataStr : String;
    ByteStr : String;
    code, code1, code2 : Integer;
    I2CByte : Byte;
    ParByte : Byte;
    lPrm    : LongInt;
begin
  for I := 0 to Count-1 do
  begin
    ucChr[Counter] := ApdComPort.GetChar;
    if ucChr[Counter] = #13 then
    begin
      CR_Bool := true;
      ApdComPort.FlushInBuffer;
    end;
    inc(Counter);
  end;
  if CR_Bool then
  begin
    for i := 0 to 1024 do
    begin
      if ucChr[i] = #13 then Break;
      DataStr   := DataStr + ucChr[i];
    end;
    Val('$'+copy(DataStr,1,2),CMD,Code);

    if (code = 0) AND GetChkSum(DataStr) then
    begin
      Case CMD of
      $72   : begin  //'r' I2C-IC nicht gefunden / gefunden , Daten werden gelesen
                ByteStr := '$'+ copy(DataStr,7,2);
                Val(ByteStr,I2CByte,Code);
                if (Code = 0) AND (I2CByte = 1) then
                begin
                  I2CReadB := true;
                  I2CReadStr := '';
                end else
                begin
                  ReadI2CTimer.Enabled := false;
                  I2CBusy  := false;
                  I2CReadB := false;
                  I2CReadStr := '';
                  ByteStr := '$'+ copy(DataStr,3,2);
                  Val(ByteStr,SlvAdr,Code1);
                  ByteStr := '$'+ copy(DataStr,5,2);
                  Val(ByteStr,SlvAdr,Code2);

                  if Code1 <> 0 then DIPSwitch := 0;
                  lPrm := DIPSwitch Shl 8;

                  if Code2 <> 0 then SlvAdr := 0;
                  lPrm := (lPrm OR SlvAdr) Shl 16;
                  SendMSG(MSGReadResponse,lPrm);
                end;
              end;
      $64   : begin   // 'd' Daten wurden gelesen
                if I2CReadB then
                begin
                  ReadI2CTimer.Enabled := false;
                  I2CReadStr := copy(DataStr,3,Length(DataStr)-4);
                  ParByte := ParseI2CData(I2CReadStr);
                  I2CReadB := false;
                  I2CBusy := false;
                  lPrm := DIPSwitch shl 8;
                  lPrm := lPrm OR SlvAdr;
                  lPrm := (lPrm Shl 16) + ParByte;
                  SendMSG(MSGReadResponse,lPrm);
                end;
              end;
      $77   : begin   //'w' Daten geschrieben
                WriteI2CTimer.Enabled := false;
                I2CBusy := false;

                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);

                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,SlvAdr,Code1);

                ByteStr := '$'+ copy(DataStr,7,2);
                Val(ByteStr,I2CByte,Code2);

                if Code <> 0 then DIPSwitch := 0;
                lPrm := DIPSwitch Shl 8;

                if Code1 <> 0 then SlvAdr := 0;
                lPrm := (lPrm OR SlvAdr) Shl 16;

                if (Code = 0) AND (Code1 = 0) AND (Code2 = 0) AND (I2CByte = 1) then
                  SendMSG(MSGWriteResponse,lPrm + 1)
                else
                  SendMSG(MSGWriteResponse,lPrm);
              end;
      $65   : begin   // 'e' einstellungen gendert
                WriteI2CTimer.Enabled := false;
                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);

                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,I2SCLH,Code1);

                ByteStr := '$'+ copy(DataStr,7,2);
                Val(ByteStr,I2SCLL,Code2);

                I2CBusy := false;
                if (Code = 0) AND (Code1 = 0) AND (Code2 = 0) AND (I2SCLH <> 0) then
                begin
                   SCLFreq := Round(MPCLK / (2*1000*(I2SCLH + I2SCLL)));
                   lPrm := DIPSwitch shl 16;
                   lPrm := lPrm OR SCLFreq;
                   SendMSG(MSGSCLChange,lPrm);
                end;
              end;
      $69   : begin    // 'i' einstellungen abgefragt
                WriteI2CTimer.Enabled := false;
                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);

                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,I2SCLH,Code1);

                ByteStr := '$'+ copy(DataStr,7,2);
                Val(ByteStr,I2SCLL,Code2);

                I2CBusy := false;
                if (Code = 0) AND (Code1 = 0) AND (Code2 = 0) AND (I2SCLH <> 0) then
                begin
                   SCLFreq := Round(MPCLK / (2*1000*(I2SCLH + I2SCLL)));
                   lPrm := DIPSwitch shl 16;
                   lPrm := lPrm OR SCLFreq;
                   SendMSG(MSGSCLFreq,lPrm);
                end;
              end;
      $6D   : begin   // 'm' IO-Pin 1
                WriteI2CTimer.Enabled := false;
                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);
                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,I2CByte,Code1);
                I2CBusy := false;
                lPrm := DIPSwitch shl 16;
                if (Code = 0) AND (Code1 = 0) AND (I2CByte = 1) then
                  SendMSG(MSGIO1,lPrm + 1)
                else
                  SendMSG(MSGIO1,lPrm);
              end;

      $6E   : begin   // 'n' IO-Pin 2
                WriteI2CTimer.Enabled := false;
                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);
                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,I2CByte,Code1);
                I2CBusy := false;
                lPrm := DIPSwitch shl 16;
                if (Code = 0) AND (Code1 = 0) AND (I2CByte = 1) then
                  SendMSG(MSGIO2,lPrm + 1)
                else
                  SendMSG(MSGIO2,lPrm);
              end;

      $6F   : begin   // 'o' Pins Abfrage
                WriteI2CTimer.Enabled := false;
                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);
                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,I2CByte,Code1);
                I2CBusy := false;
                lPrm := DIPSwitch shl 16;
                if (Code = 0) AND (Code1 = 0) then
                  SendMSG(MSGIO,lPrm OR I2CByte);
              end;

      $63   : begin        // 'c' Check SlvAdr
                WriteI2CTimer.Enabled := false;
                I2CBusy := false;

                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);

                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,SlvAdr,Code1);

                ByteStr := '$'+ copy(DataStr,7,2);
                Val(ByteStr,I2CByte,Code2);

                lPrm := DIPSwitch shl 8;
                lPrm := (lPrm OR SlvAdr) shl 16;

                if (Code = 0) AND (Code1 = 0) AND (Code2 = 0) AND (I2CByte = 1) then
                  SendMSG(MSGCSlave,lPrm + 1)
                else
                  SendMSG(MSGCSlave,lPrm);
              end;

      $76   : begin  // 'v' Versionsnummer
                WriteI2CTimer.Enabled := false;
                DevVer  := PChar(copy(DataStr,5,length(DataStr)-6));
                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);
                if Code <> 0 then DIPSwitch := 0;
                I2CBusy := false;
                SendMSG(MSGDVer,DIPSwitch);
              end;

      $73   : begin  // 's' Checksum-Fehler;
                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);
                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,I2CByte,Code1);
                if (Code = 0) AND (Code1 = 0) AND (I2CByte = 1) then
                  SendMSG(MSGCSErr,DIPSwitch);
              end;
      $00FF : begin
                ByteStr := '$'+ copy(DataStr,3,2);
                Val(ByteStr,DIPSwitch,Code);
                ByteStr := '$'+ copy(DataStr,5,2);
                Val(ByteStr,I2CByte,Code1);
                if (Code = 0) AND (Code1 = 0) AND (I2CByte = 0) then
                SendMSG(MSGUnknownCMD,DIPSwitch);
              end;
      end;
    end;
    ucChr   := '';
    Counter := 0;
    CR_Bool := false;
  end;
end;

Function Connect(State : Byte) : Byte; stdcall;
begin
  State := State AND $01;
  Try
    Try
      ApdComport.Open := Bool(State);
    except
      MessageDlg('Can not open COMPORT!', mtERROR,[mbOk], 0);
    end;
  Finally
    if ApdComport.Open then
      Result := ApdComport.ComNumber
    else Result := 0;
  end;
end;

Function IsConnected : Byte; stdcall;
begin
  Result := Integer(ApdComport.Open);
end;

function GetComPort : Word; stdcall;
begin
  Result := ApdComport.ComNumber;
end;

Function SetComPort(COMNr : Word) : Word; stdcall;
begin
  ApdComport.ComNumber := COMNr;
  Result := ApdComport.ComNumber;
end;

function ReadI2C(pConAdr, pSlvAdr, Cnt : Byte) : Byte; stdcall;
begin
  if Not I2CBusy then
  begin
    if not SendStr(IntToHex(pConAdr,2)+I2Cr+IntToHex(pSlvAdr,2)+IntToHex(Cnt,2)) then
      Result := 1  // ERROR: ComPort is closed
    else begin
       I2CBusy := true;
       SlvAdr  := pSlvAdr;
       ReadI2CTimer.Enabled := true;
       Result := 0;  //OK
    end;
  end else
    Result := 2; // I2C-Bus Busy
end;

function WriteI2C(pConAdr, pSlvAdr : Byte; iData: Array of Byte): Byte; stdcall
var str  : String;
    iLen,i : Byte;
begin
  iLen := Length(iData) - 1;
  if Not I2CBusy then
  begin
    Str := '';
    Str := IntToHex(pConAdr,2);
    Str := Str + I2Cw;
    Str := Str + IntToHex(pSlvAdr,2);
    for i := 0 to iLen do
      Str := Str + IntToHex(iData[i],2);
    if SendStr(Str) then
    begin
      I2CBusy := true;
      SlvAdr  := pSlvAdr;
      WriteI2CTimer.Enabled := true;
      Result  := 0; // OK
    end else
      Result := 1; // ERROR: ComPort is closed;
  end else Result := 2; // I2C-Bus Busy
end;

function WriteI2CPCHAR(pConAdr, pSlvAdr : Byte; pcData : PChar) : Byte; stdcall;
var stri : string;
    iLen : Byte;
    i    : Integer;
begin
  iLen := Length(pcData);
  if Not I2CBusy then
  begin
    Stri := IntToHex(pConAdr,2);
    Stri := Stri + I2Cw;
    stri := stri + IntToHex(pSlvAdr,2);
    i := 1;
    While i <= iLen - 1 do
    begin
      stri := stri + Copy(pcData,i,2);
      i := i + 2;
    end;
    if SendStr(stri) then
    begin
      I2CBusy := true;
      SlvAdr  := pSlvAdr;
      WriteI2CTimer.Enabled := true;
      Result := 0; //OK
    end else Result := 1; // ERROR: Comport is closed
  end else Result := 2; // I2C-Busy Busy
end;

function GetI2CData : TI2CData; stdcall;
begin
  Result := I2CData;
end;

function SetSCLCLK(pConAdr : Byte; SCLCLK : Word) : Byte; stdcall;
var pCLK : Byte;
    rCLK : Integer;
begin
  If (SCLCLK > 400) OR (SCLCLK < 12) then
    Result := 3
  else begin
    if Not I2CBusy then
    begin
      rCLK := round(2 * Frac(MPCLK / (4*SCLCLK*1000)));
      pCLK := MPCLK DIV (4*SCLCLK*1000);
      if SendStr(IntToHex(pConAdr,2)+I2CFreqChange+IntToHex(pCLK,2)+IntToHex(rCLK+pCLK,2)) then
      begin
        WriteI2CTimer.Enabled := true;
        I2CBusy := true;
        Result := 0;
      end else Result := 1;
    end else Result := 2;
  end;
end;

function ReSetIO(pConAdr, Pin, State : Byte) : Byte; stdcall;
var str  : String;
begin
  State := State AND $01;
  Pin   := Pin AND $01;
  if Not I2CBusy then
  begin
    Str := '';
    Str := IntToHex(pConAdr,2);
    if Pin = 0 then Str := Str + I2CIO1
    else Str := Str + I2CIO2;
    Str := Str + IntToHex(State,2);
    if SendStr(Str) then
    begin
      WriteI2CTimer.Enabled := true;
      I2CBusy := true;
      Result := 0;
    end else Result := 1;
  end else Result := 2;
end;

function RequestIO(pConAdr : Byte) : Byte;
var str  : String;
begin
  if Not I2CBusy then
  begin
    Str := '';
    Str := IntToHex(pConAdr,2);
    Str := Str + I2CIO;
    if SendStr(Str) then
    begin
      WriteI2CTimer.Enabled := true;
      I2CBusy := true;
      Result := 0;
    end else Result := 1;
  end else Result := 2;
end;

function GetVersion : PChar; stdcall;
begin
  Result := DevVer;
end;

function RequestVersion(pConAdr : Byte) : Byte; stdcall;
begin
  if Not I2CBusy then
  begin
    if not SendStr(IntToHex(pConAdr,2)+I2CVerNr) then
      Result := 1  // ERROR: ComPort is closed
    else begin
       WriteI2CTimer.Enabled := true;
       I2CBusy := true;
       Result := 0;  //OK
    end;
  end else
    Result := 2; // Busy
end;

function RequestSCLFreq(pConAdr : Byte) : Byte; stdcall;
begin
  if Not I2CBusy then
  begin
    if not SendStr(IntToHex(pConAdr,2)+I2CFreqReq) then
      Result := 1  // ERROR: ComPort is closed
    else begin
       WriteI2CTimer.Enabled := true;
       I2CBusy := true;
       Result := 0;  //OK
    end;
  end else
    Result := 2; // Busy
end;

function CheckSlvAdr(pConAdr, pSlvAdr : Byte) : Byte; stdcall;
var Stri : String;
begin
  if (pSlvADr AND $01 = $01) then
    Result := 3
  else begin
    if Not I2CBusy then
    begin
      Stri := IntToHex(pConAdr,2);
      Stri := Stri + I2CCheck;
      Stri := Stri + IntToHex(pSlvAdr,2);
      if not SendStr(Stri) then
        Result := 1  // ERROR: ComPort is closed
      else begin
         WriteI2CTimer.Enabled := true;
         I2CBusy := true;
         Result := 0;  //OK
      end;
    end else
      Result := 2; // Busy
  end;    
end;

procedure NewExit;
begin
  //ExitProc := SaveExit;
  //showMessage('Exitprozedur der DLL');
  { DLL wird entladen }
end;

exports
   SerI2CCreate,
   SerI2CDestroy,
   Connect,
   IsConnected,
   GetComPort,
   SetComPort,
   ReadI2C,
   WriteI2C,
   WriteI2CPCHAR,
   SetReadTOut,
   SetWriteTOut,
   GetReadTOut,
   GetWriteTOut,
   GetI2CData,
   SetSCLCLK,
   ReSetIO,
   GetVersion,
   RequestIO,
   RequestVersion,
   RequestSCLFreq,
   CheckSlvAdr;
begin
//  AddExitProc(NewExit);

 // SaveExit := ExitProc;
 // ExitProc := @NewExit;
end.
